/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 *
 * Created by Tommaso Latini <ipapalini@domotz.com> on 17/09/15.
 */

const BASE_RETRY_INTERVAL = 10000;
const MAX_RETRY_INTERVAL = 60 * 1000 * 5; // 5 minutes

var factory = function (connection, consumer, channel, resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs()
    var proc = resourceLocator.utils.proc;
    var status = resourceLocator.status;

    var retry_interval = BASE_RETRY_INTERVAL;

    function retryConnection() {
        myConsole.info('Retrying connection in ' + retry_interval + ' ms');
        setTimeout(function () {
            o.listen();
        }, retry_interval);
    }

    function setAgentStatus() {
        if (status.get() === status.RESTRICTED) {
            status.set(status.CONFIGURED);
        }
        retry_interval = BASE_RETRY_INTERVAL;
    }

    function setEventHandler(conn) {
        conn.on('error', function(err){
            myConsole.error('Connection Error ' + err.message);
        });
        conn.on('close', function(err){
            myConsole.error('Connection Close ' + err.message);
        });
        return conn;
    }

    function listen() {

        var url = connection.createConnectionUrl();
        connection.connect(url)
            .then(setEventHandler)
            .then(connection.setSignalHandler)
            .then(channel.create)
            .then(channel.setEventHandler)
            .then(consumer.consume)
            .then(setAgentStatus)
            .catch(function (err) {
                myConsole.error('Connection refused with error: ' + err.message);
                if (err.message.indexOf("403") > -1) {
                    status.set(status.RESTRICTED);
                    if (retry_interval < MAX_RETRY_INTERVAL) {
                        retry_interval += BASE_RETRY_INTERVAL;
                    }
                }
                myConsole.error(err.stack);
                o.retryConnection();
            });
    }


    var o = {
        retryConnection: retryConnection,
        listen: listen
    };
    return o;
};


module.exports.factory = factory;