/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz UK LLP
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 *
 * Created by Mohamed Kallel <mohamed.kallel@iovision.io> on 08/04/2020.
 * This is used for checking blocked nmap processes more than 15 minutes
 *
 */
var exec = require('child_process').exec;
var q = require('q');

var taskToPromise = function (command) {
    var deferred = q.defer();
    exec(command, function (err, stdout, stderr) {
        if (err || stderr) {
            return deferred.resolve('');
        }
        deferred.resolve(stdout);
    });
    return deferred.promise;
};


var cleanBlockedProcess = function () {
    // this cleanup code might be related to https://github.com/nmap/nmap/issues/2104 (fixed in nmap 7.90)
    console.info('HANDLER - Start cleaning blocked processes');
    return taskToPromise('WMIC PROCESS GET NAME, CREATIONDATE, ProcessID | findstr /i "domotz_nmap.exe domotz_bash.exe"')
        .then(function (res) {
            // example of output
            // 20200408211724.960575+120  domotz_nmap.exe
            if (res) {
                const pids = [];
                const lines = res.split('\n');
                lines.forEach(function (line) {
                    console.debug('Handler - Process found: ' + line);
                    const year = line.substring(0, 4);
                    const month = line.substring(4, 6);
                    const day = line.substring(6, 8);
                    const hour = line.substring(8, 10);
                    const minutes = line.substring(10, 12);
                    const seconds = line.substring(12, 14);
                    const timestamp = new Date(year + '-' + month + '-' + day + 'T' + hour + ':' + minutes + ':' + seconds).getTime();
                    const diff = new Date().getTime() - timestamp;
                    if (diff > 15 * 60 * 1000) {
                        const pid = line.split(/\s+/)[2];
                        pids.push(pid);
                    }
                });
                return pids;
            }
            return [];
        }).then(function (pids) {
            console.info('HANDLER - pid to clean: ' + pids);
            if (!pids.length) {
                return null;
            }
            const promises = [];
            pids.forEach(function (pid) {
                promises.push(taskToPromise('taskkill /PID ' + pid + ' /F'));
            });
            return q.all(promises);
        }).then(function () {
            var message = 'HANDLER - Cleaning blocked processes ended';
            console.info(message);
            return message;
        }).catch(function (err) {
            return err;
        });
};

module.exports.handler = cleanBlockedProcess;