/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Iacopo Papalini <iacopo@domotz.com> on 04/08/2017.
 */

const MAIN_INTERFACE = "eth0";


function ipWrapper(resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();
    var proc = resourceLocator.utils.proc;
    var lodash = resourceLocator.lodash;
    var q = resourceLocator.q;

    var ipCall = function (tool, cmd, options) {
        var ipCommand = ['ip', tool, cmd];

        if (options.constructor === Array) {
            ipCommand = ipCommand.concat(options);
        } else {
            for (var key in options) {
                if (options.hasOwnProperty(key)) {
                    ipCommand.push(key);
                    ipCommand.push(options[key]);
                }
            }
        }

        var ipCmd = ipCommand.join(' ');

        myConsole.debug("Executing ip command: " + ipCmd);

        return proc.exec(ipCmd);
    };

    function _flush() {
        return ipCall('addr', 'flush', {dev: MAIN_INTERFACE});
    }

    function activate_dhcp() {
        "use strict";
        return proc.spawn('/sbin/dhclient', ['-i', MAIN_INTERFACE]);
    }

    function ping(ip, count, timeout) {
        "use strict";
        return proc.spawn('ping', [ip, '-c', count, '-t', timeout]).then(
            function (data) {
                var lost_packets = /^.+, (\d+)% packet loss,/gm.exec(data);

                return {"lost_packets_ratio": parseFloat(lost_packets[1])};
            }
        ).catch(function (error) {
                return {"error": error};
            });
    }

    function _parseVlan(raw_output) {
        var links = [];

        var output = raw_output.split('\n');

        for (var line = 0, output_length = output.length - 1; line < output_length; line += 2) {
            var link_line = output[line];
            var link_fields = link_line.trim().split(/\s/g);
            var wasDeleted = link_fields[0] === 'Deleted';
            if (wasDeleted) {
                link_fields.shift();
            }

            var name = link_fields[1].split(':')[0];
            if (name.split('@').length === 2) {
                links.push(name.split('@')[0]);
            }
        }
        return links;
    }

    function _getVlansIP(vlanNames) {
        var promises = [];
        lodash.forOwn(vlanNames, function (name) {
            var deferred = q.defer();
            ipCall('addr', 'show', ['dev', name]).then(function (raw_output) {
                var output = raw_output.split('\n');
                var addrLine = output[2];
                var address = addrLine.trim().split(/\s/g)[1];

                deferred.resolve({name: name, address: address});
            });
            promises.push(deferred.promise);
        });
        return q.all(promises);
    }

    function retrieveVlans() {
        return ipCall('link', 'show', {}).then(_parseVlan).then(_getVlansIP)
    }

    return {
        call: ipCall,
        flush: _flush,
        activate_dhcp: activate_dhcp,
        ping: ping,
        retrieveVlans: retrieveVlans,
        _parseVlan: _parseVlan
    };
}


module.exports.ipWrapper = ipWrapper;