/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Iacopo Papalini <iacopo@domotz.com>
 */

var request = require('request');
const util = require('util');


function createDevice(device, driverName, agentDriverSettings, myConsole) {
    function buildRequest(options){
        if (typeof options !== 'object') {
            options = {url: 'http://' + device.ip + options};
        } else {
            var port = '';
            if (options.port) {
                port = ':' + options.port;
            }
            var password='';
            if (device.credentials) {
                password = new Buffer(device.credentials.password, 'base64').toString();
                options.url = options.url.replace('${username}', device.credentials.username);
                options.url = options.url.replace('${password}', password);
            }

            if (options.auth === 'basic' && device.credentials) {
                myConsole.debug(device.credentials);
                options.auth = {
                    'user': device.credentials.username,
                    'pass': password,
                    'sendImmediately': true
                };
            }
            else {
                options.auth = undefined;
            }
            options.url = 'http://' + device.ip + port + options.url;
        }
        myConsole.debug(util.inspect(options));
        return options;
    }

    return {
        ip: function () {
            return device.ip;
        },
        createSNMPSession: function () {
            return require('./snmp').createSessionForDevice(device);
        },
        http: {
            get: function (options, callback) {
                myConsole.info("Performing GET towards:" + options.url);
                options = buildRequest(options);
                request.get(options, callback);
            },
            post: function (options, callback) {
                myConsole.info("Performing POST towards:" + options.url);
                options = buildRequest(options);
                request.post(options, callback);
            }
        },
        createVariable: function (id, name, value, unit) {
            if (id === null || id === undefined || id.length < 1 || id.length > agentDriverSettings.max_var_id_len)
            {
                throw Error("Invalid variable id: " + id);
            }
            if (unit){
                unit = unit.substr(0, agentDriverSettings.max_var_unit_len);
            } else {
                unit = null
            }
            if (value == undefined){
                value = null
            } else {
                value = String(value)
            }
            return {
                "uid": id,
                "unit": unit,
                "value": value,
                "label": name,
                "driver_name": driverName
            };
        }
    };
}

module.exports.device = createDevice;