/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Iacopo Papalini <iacopo@domotz.com>
 */
const DEFAULT_TIMEOUT = 5000;

const DEFAULT_CUSTOM_DRIVER_SETTINGS = {
    max_log_entries: 100,
    max_var_id_len: 50,
    max_var_per_device: 100,
    max_var_unit_len: 10,
    max_data_len: 4096
};

var path = require('path');

function createMessageListener(subProcess, onError, onSuccess) {
    return function (response) {
        console.debug("SANDBOX Response arrived: ", response.outcome);
        console.debug("SANDBOX internal log:" + response.myLog);
        if (response.variables) {
            console.debug("SANDBOX Response contained variables: " + JSON.stringify(response.variables));
        }

        subProcess.kill();
        subProcess = null;
        if (response.outcome === 'success') {
            return onSuccess(response);
        } else {
            return onError(response);
        }
    };
}

function createErrorListener(subProcess, onError) {
    return function () {
        console.debug("SANDBOX Process exited");
        if (subProcess !== null) {
            console.error("SANDBOX Sandbox exited without sending back message - error");
            return onError(Error("Sandbox exited without sending back message"));
        }
    };
}

function sandboxRunner(data, resourceLocator, onError, onSuccess) {
    var subProcess = require('child_process').fork(__dirname + path.sep + 'sandbox.js');
    var scriptText = data.code;
    var device = data.device;
    var driverName = data.driver_name;
    var timeout = data.timeout || DEFAULT_TIMEOUT;
    var logLevel = data.logLevel || 'warning';

    if (!scriptText || !device) {
        return onError(Error("Missing script text or device: " + JSON.stringify(data)));
    }
    if (!device.ip) {
        device.ip = resourceLocator.interfacesBindingStorage.getIpsFromMac(device.hw_address)[0];
    }

    console.debug("SANDBOX Running %s", scriptText);
    console.debug("SANDBOX device: %s", JSON.stringify(device));

    subProcess.on('message', createMessageListener(subProcess, onError, onSuccess));
    subProcess.on('exit', createErrorListener(subProcess, onError));

    subProcess.send({
        script: scriptText.toString(),
        device: device,
        driverName: driverName,
        timeout: timeout,
        logLevel: logLevel,
        agentDriverSettings: resourceLocator.configuration.settings.custom_driver || DEFAULT_CUSTOM_DRIVER_SETTINGS
    });
}

module.exports.sandboxRunner = sandboxRunner;
